#include <gtk/gtk.h>
#include "spawn.h"
#include <signal.h>
#include <unistd.h>
#include <glade/glade.h>
#include <string.h>
#include <stdlib.h>

static GladeXML *xml;

static void
build_gui (void)
{
    GtkWidget *window;
    
    const char *glade_file = "./benchmark.glade";
    
    xml = glade_xml_new (glade_file, NULL, NULL);

    window = glade_xml_get_widget (xml, "window1");

    gtk_widget_show_all (window);
    gtk_window_resize (GTK_WINDOW (window), 700, 500);

    g_signal_connect (window, "delete_event", G_CALLBACK (gtk_main_quit), NULL);
}

#define ITERATIONS 100
#define STEP 50

static void
run_benchmark (void)
{
    int i;
    
    GtkWidget *w = glade_xml_get_widget (xml, "hpaned1");
    GtkPaned *paned = GTK_PANED (w);
    GTimer *timer;
    double elapsed;

    gtk_paned_set_position (paned, 350);

    while (g_main_pending ())
	g_main_iteration(FALSE);
    gdk_flush();
    
    timer = g_timer_new ();
    
    for (i = 0; i < ITERATIONS; ++i)
    {
	int j;

	for (j = 100; j < 600; j += STEP)
	{
	    gtk_paned_set_position (paned, j);
	    
	    while (g_main_pending ())
		g_main_iteration(FALSE);
	}

	for (j = 600; j >= 100; j -= STEP)
	{
	    gtk_paned_set_position (paned, j);

	    while (g_main_pending ())
		g_main_iteration(FALSE);
	}
    }

    while (g_main_pending ())
	g_main_iteration(FALSE);
    gdk_flush();
    
    elapsed = g_timer_elapsed (timer, NULL);

    g_print ("elapsed time: %.3f  (%.3f resizes per second)\n",
	     elapsed, (ITERATIONS * (2 * (500.0 / STEP))) / elapsed);
}

int
main (int argc, char *argv[])
{
    gtk_init (&argc, &argv);
    
    build_gui ();

    run_benchmark ();
    
    gtk_main ();
    return 0;
}
